#generates graphs that are used in unit tests

from codebase import mixed_graph as mixed

def generate_graphs():
    nodes = ['X1','X2','X3','X4','X5','X6']

    G1 = mixed.LabelledMixedGraph()
    G2 = mixed.LabelledMixedGraph()
    G3 = mixed.LabelledMixedGraph()
    G4 = mixed.LabelledMixedGraph()
    G5 = mixed.LabelledMixedGraph()
    G6 = mixed.LabelledMixedGraph()

    empty = mixed.LabelledMixedGraph()

    for i in nodes:
        G1.add_node(i)
        G2.add_node(i)
        G3.add_node(i)
        G4.add_node(i)
        G5.add_node(i)
        G6.add_node(i)
        empty.add_node(i)

    #simple causal chain
    G1.add_directed('X1', 'X2')
    G1.add_directed('X2', 'X3')
    G1.add_directed('X3', 'X4')
    G1.add_directed('X4', 'X5')
    G1.add_directed('X5', 'X6')

    #graph with bidirected edges
    G2.add_directed('X1', 'X2')
    G2.add_directed('X2', 'X3')
    G2.add_directed('X3', 'X4')
    G2.add_bidirected('X4', 'X5')
    G2.add_bidirected('X5', 'X6')
    G2.add_bidirected('X1', 'X4')

    G3.add_directed('X1', 'X2')
    G3.add_directed('X2', 'X3')
    G3.add_directed('X3', 'X4')
    G3.add_directed('X5', 'X4')
    G3.add_directed('X5', 'X6')

    # simple undirected chain
    G4.add_undirected('X1', 'X2')
    G4.add_undirected('X2', 'X3')
    G4.add_undirected('X3', 'X4')
    G4.add_undirected('X4', 'X5')
    G4.add_undirected('X5', 'X6')

    # cyclic graph
    G5.add_directed('X1', 'X2')
    G5.add_directed('X2', 'X3')
    G5.add_directed('X3', 'X1')
    G5.add_directed('X6', 'X5')
    G5.add_directed('X5', 'X6')

    # inverse chain
    G6.add_directed('X2', 'X1')
    G6.add_directed('X3', 'X2')
    G6.add_directed('X4', 'X3')
    G6.add_directed('X5', 'X4')
    G6.add_directed('X6', 'X5')

    #smaller graphs

    G7 = mixed.LabelledMixedGraph()
    G8 = mixed.LabelledMixedGraph()

    nodes2 = ['X1', 'X2', 'X3', 'X4']
    for i in nodes2:
        G7.add_node(i)
        G8.add_node(i)

    G7.add_directed('X1', 'X2')
    G7.add_directed('X1', 'X3')
    G7.add_directed('X1', 'X4')
    G7.add_directed('X2', 'X3')
    G7.add_directed('X2', 'X4')
    G7.add_directed('X3', 'X4')




    return {'chain': G1,'with_bidirected': G2,'simple_collider': G3,'undirected_chain': G4,'cyclic': G5, 'inverse_chain': G6, 'empty': empty, 'full_4_nodes': G7, 'empty_4_nodes': G8 }





from codebase import mixed_graph as mixed
from codebase import metrics
import json
import pytest
from ..tests import Graphs_for_testing as G_testing

@pytest.fixture
def test_graphs():
    graphs = G_testing.generate_graphs()
    return {
        'G1': graphs['simple_collider'],
        'G2': graphs['chain'], 
        'G3': graphs['empty']
    }

# Load ground truth values
def load_ground_truth():
    with open('tests/test_answers/test_sc_metric_DAGs_answers.json', 'r') as f:
        return json.load(f)

ground_truth = load_ground_truth()

def test_sc_metric_DAGs_cases(test_graphs):
    G1 = test_graphs['G1']
    G2 = test_graphs['G2']
    G3 = test_graphs['G3']

    assert metrics.metric_DAGs(G3,G1,type ='sc', max_order=None, randomize_higher_order = 0, normalized= True) == ground_truth['metric_DAGs(empty,G1,type =sc, max_order=None, randomize_higher_order = 0, normalized= True)']
    assert metrics.metric_DAGs(G1,G1,type ='sc', max_order=None, randomize_higher_order = 0, normalized= True) == ground_truth['metric_DAGs(G1,G1,type =sc, max_order=None, randomize_higher_order = 0, normalized= True)']
    
    assert metrics.metric_DAGs(G3,G1,type ='sc', max_order=2, randomize_higher_order = 0, normalized= True) == ground_truth['metric_DAGs(empty,G1,type =sc, max_order=2, randomize_higher_order = 0, normalized= True)']
    assert metrics.metric_DAGs(G1,G1,type ='sc', max_order=2, randomize_higher_order = 0, normalized= True) == ground_truth['metric_DAGs(G1,G1,type =sc, max_order=2, randomize_higher_order = 0, normalized= True)']
    
    assert metrics.metric_DAGs(G3,G1,type ='sc', max_order=2, randomize_higher_order = 100, normalized= True) == ground_truth['metric_DAGs(empty,G1,type =sc, max_order=2, randomize_higher_order = 100, normalized= True)']
    assert metrics.metric_DAGs(G1,G1,type ='sc', max_order=2, randomize_higher_order = 100, normalized= True) == ground_truth['metric_DAGs(G1,G1,type =sc, max_order=2, randomize_higher_order = 100, normalized= True)']
    
    assert metrics.metric_DAGs(G3,G1,type ='sc', max_order=2, randomize_higher_order = 100, normalized= False) == ground_truth['metric_DAGs(empty,G1,type =sc, max_order=2, randomize_higher_order = 100, normalized= False)']
    assert metrics.metric_DAGs(G1,G1,type ='sc', max_order=2, randomize_higher_order = 100, normalized= False) == ground_truth['metric_DAGs(G1,G1,type =sc, max_order=2, randomize_higher_order = 100, normalized= False)']

    assert metrics.metric_DAGs(G2,G1,type ='sc', max_order=None, randomize_higher_order = 0, normalized= True) == ground_truth['metric_DAGs(G2,G1,type =sc, max_order=None, randomize_higher_order = 0, normalized= True)']
    assert metrics.metric_DAGs(G1,G2,type ='sc', max_order=None, randomize_higher_order = 0, normalized= True) == ground_truth['metric_DAGs(G1,G2,type =sc, max_order=None, randomize_higher_order = 0, normalized= True)']





def string_graph_to_mixed_graph(graph):
    '''Converts a graph in the format of the Tigramite package to the LabelledMixedGraph class'''
    G = mixed.LabelledMixedGraph()
    for i in range(graph.shape[0]):
        G.add_node(i)
    for (i, j) in zip(*np.where(graph != '')):
        if graph[i, j] == '-->':
            G.add_directed(i,j)
        if graph[i, j] == '<->':
            G.add_bidirected(i,j)
        if graph[i, j] == '---':
            G.add_undirected(i,j)
    return G


from codebase import mixed_graph as mixed
import json
import pytest
from ..tests import Graphs_for_testing as G_testing

@pytest.fixture
def test_graphs():
    graphs = G_testing.generate_graphs()
    return {
        'G1': graphs['chain'],
        'G2': graphs['simple_collider']
    }


# Load ground truth values
def load_ground_truth():
    with open('tests/test_answers/test_is_d_separated_answers.json', 'r') as f:
        return json.load(f)

ground_truth = load_ground_truth()



def test_G1_is_d_separated(test_graphs):
    G1 = test_graphs['G1']
    assert G1.is_d_separated(x={'X1',},y={'X5',},z=set(),DAG_check=False) == ground_truth["G1.is_d_separated(x={'X1',},y={'X5',},z=set(),DAG_check=False)"]
    assert G1.is_d_separated(x={'X1',},y={'X5',},z={'X4'},DAG_check=True) == ground_truth["G1.is_d_separated(x={'X1',},y={'X5',},z={'X4'},DAG_check=True)"]
    assert G1.is_d_separated(x={'X1',},y={'X5',},z={'X4','X3'},DAG_check=True) == ground_truth["G1.is_d_separated(x={'X1',},y={'X5',},z={'X4','X3'},DAG_check=True)"]


def test_G2_is_d_separated(test_graphs):
    G2 = test_graphs['G2']
    assert G2.is_d_separated(x={'X1',},y={'X5',},z=set(),DAG_check=False) == ground_truth["G2.is_d_separated(x={'X1',},y={'X5',},z=set(),DAG_check=False)"]
    assert G2.is_d_separated(x={'X1',},y={'X5',},z={'X4'},DAG_check=True) == ground_truth["G2.is_d_separated(x={'X1',},y={'X5',},z={'X4'},DAG_check=True)"]
    assert G2.is_d_separated(x={'X1',},y={'X5',},z={'X4','X3'},DAG_check=True) == ground_truth["G2.is_d_separated(x={'X1',},y={'X5',},z={'X4','X3'},DAG_check=True)"]



from codebase import mixed_graph as mixed

from ..tests import Graphs_for_testing as G_testing

print('We first test a graph that is already a DAG')

test_graphs = G_testing.generate_graphs()

G1 = test_graphs['simple_collider']

print('Graph:')
print('nodes:', G1.nodes)
print(G1)

print('Acyclification:')
print('nodes:', G1.nodes)
print(G1.get_acyclification())

G2 = test_graphs['with_bidirected']

print('Graph with bidirected edges:')
print('nodes:', G2.nodes)
print(G2)

G2_dir = G2.get_canonical_directed_graph()
print('canonical directed:')
print('nodes:', G2_dir.nodes)
print(G2_dir)

print('Acyclification:')
print('nodes:', G2_dir)
print(G2_dir.get_acyclification())




G3 = test_graphs['cyclic']

print('cyclic graph:')
print('nodes:', G3.nodes)
print(G3)

G3_acyclic = G3.get_acyclification()

print('Acyclification:')
print('nodes:', G3_acyclic.nodes)
print(G3_acyclic)



from codebase import mixed_graph as mixed

from ..tests import Graphs_for_testing as G_testing

test_graphs = G_testing.generate_graphs()


G1 = test_graphs['simple_collider']

G2 = test_graphs['chain']

G1.get_Markov_blankets()
G2.get_Markov_blankets()


print('G1:')
print(G1)
print('Markov blankets:')
print(G1.Markov_blankets)

print('G2:')
print(G2)
print('Markov blankets:')
print(G2.Markov_blankets)


import json
import pytest

from codebase import mixed_graph as mixed
from codebase import metrics

from ..tests import Graphs_for_testing as G_testing

@pytest.fixture
def test_graphs():
    graphs = G_testing.generate_graphs()
    return {
        'G1': graphs['with_bidirected'],
        'G2': graphs['chain'], 
        'G3': graphs['empty']
    }

# Load ground truth values
def load_ground_truth():
    with open('tests/test_answers/test_sc_metric_mixed_graphs_answers.json', 'r') as f:
        return json.load(f)

ground_truth = load_ground_truth()

def test_sc_metric_mixed_graphs(test_graphs):
    G1 = test_graphs['G1']
    G2 = test_graphs['G2']
    G3 = test_graphs['G3']

    # Test various configurations of the metric_mixed_graphs function
    result = metrics.metric_mixed_graphs(G3, G1, type='sc', max_order=None, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_mixed_graphs(empty, G1, type=sc, max_order=None, randomize_higher_order=0, normalized=True)']

    result = metrics.metric_mixed_graphs(G3, G1, type='s', max_order=None, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_mixed_graphs(empty, G1, type=s, max_order=None, randomize_higher_order=0, normalized=True)']

    result = metrics.metric_mixed_graphs(G3, G1, type='c', max_order=None, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_mixed_graphs(empty, G1, type=c, max_order=None, randomize_higher_order=0, normalized=True)']



from codebase import mixed_graph as mixed

from ..tests import Graphs_for_testing as G_testing


test_graphs = G_testing.generate_graphs()

G1 = test_graphs['chain']



print('nodes:', G1.nodes)
print ('graph:')
print(G1)

print('Node pair:')
print('X1, X2')
print('ZL-separator:', G1.find_minimal_d_separator({'X1',},{'X2',}))

print('Node pair:')
print('X2, X1')
print('ZL-separator:', G1.find_minimal_d_separator({'X2',},{'X1',}))

print('Node pair:')
print('X1, X3')
print('ZL-separator:', G1.find_minimal_d_separator({'X1',},{'X3',}))

print('Node pair:')
print('X3, X1')
print('ZL-separator:', G1.find_minimal_d_separator({'X3',},{'X1',}))

print('Node pair:')
print('X1, X4')
print('ZL-separator:', G1.find_minimal_d_separator({'X1',},{'X4',}))

print('alternative:')

import networkx as nx

print(nx.find_minimal_d_separator(G1.nx_graph,{'X1',},{'X4',}))



from codebase import mixed_graph as mixed

from ..tests import Graphs_for_testing as G_testing

test_graphs = G_testing.generate_graphs()


G1 = test_graphs['simple_collider'].get_CPDAG()
G1.get_representative_of_MEC()

print('CPDAG:')
print('nodes:', G1.nodes)
print(G1)
print('Representative:')
print(G1._representative)


G2 = test_graphs['chain'].get_CPDAG()
G2.get_representative_of_MEC()

print('CPDAG:')
print('nodes:', G2.nodes)
print(G2)
print('Representative:')
print(G2._representative)

G3 = test_graphs['empty'].get_CPDAG()
G3.get_representative_of_MEC()

print('CPDAG:')
print('nodes:', G3.nodes)
print(G3)
print('Representative:')
print(G3._representative)



import pytest
import json
from codebase import metrics
from ..tests import Graphs_for_testing as G_testing

@pytest.fixture
def test_graphs():
    graphs = G_testing.generate_graphs()
    return {
        'G1': graphs['simple_collider'],
        'G2': graphs['chain'],
        'G3': graphs['empty']
    }

# Load ground truth values
def load_ground_truth():
    with open('tests/test_answers/test_c_metric_DAGs_answers.json', 'r') as f:
        return json.load(f)

ground_truth = load_ground_truth()

def test_metric_dags_empty_G1_base(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G3'], test_graphs['G1'], type='c', 
                               max_order=None, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_DAGs(empty,G1,type =c, max_order=None, randomize_higher_order = 0, normalized= True)']

def test_metric_dags_G1_G1_base(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G1'], test_graphs['G1'], type='c', 
                               max_order=None, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_DAGs(G1,G1,type =c, max_order=None, randomize_higher_order = 0, normalized= True)']

def test_metric_dags_empty_G1_order2(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G3'], test_graphs['G1'], type='c', 
                               max_order=2, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_DAGs(empty,G1,type =c, max_order=2, randomize_higher_order = 0, normalized= True)']

def test_metric_dags_G1_G1_order2(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G1'], test_graphs['G1'], type='c', 
                               max_order=2, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_DAGs(G1,G1,type =c, max_order=2, randomize_higher_order = 0, normalized= True)']

def test_metric_dags_empty_G1_order2_rand100(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G3'], test_graphs['G1'], type='c', 
                               max_order=2, randomize_higher_order=100, normalized=True)
    assert result == ground_truth['metric_DAGs(empty,G1,type =c, max_order=2, randomize_higher_order = 100, normalized= True)']

def test_metric_dags_G1_G1_order2_rand100(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G1'], test_graphs['G1'], type='c', 
                               max_order=2, randomize_higher_order=100, normalized=True)
    assert result == ground_truth['metric_DAGs(G1,G1,type =c, max_order=2, randomize_higher_order = 100, normalized= True)']

def test_metric_dags_empty_G1_order2_rand100_unnorm(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G3'], test_graphs['G1'], type='c', 
                               max_order=2, randomize_higher_order=100, normalized=False)
    assert result == ground_truth['metric_DAGs(empty,G1,type =c, max_order=2, randomize_higher_order = 100, normalized= False)']

def test_metric_dags_G1_G1_order2_rand100_unnorm(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G1'], test_graphs['G1'], type='c', 
                               max_order=2, randomize_higher_order=100, normalized=False)
    assert result == ground_truth['metric_DAGs(G1,G1,type =c, max_order=2, randomize_higher_order = 100, normalized= False)']

def test_metric_dags_G2_G1(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G2'], test_graphs['G1'], type='c', 
                               max_order=None, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_DAGs(G2,G1,type =c, max_order=None, randomize_higher_order = 0, normalized= True)']

def test_metric_dags_G1_G2(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G1'], test_graphs['G2'], type='c', 
                               max_order=None, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_DAGs(G1,G2,type =c, max_order=None, randomize_higher_order = 0, normalized= True)']

def test_metric_dags_G1_empty(test_graphs):
    result = metrics.metric_DAGs(test_graphs['G1'], test_graphs['G3'], type='c', 
                               max_order=2, randomize_higher_order=0, normalized=True)
    assert result == ground_truth['metric_DAGs(G1,empty,type =c, max_order=2, randomize_higher_order = 0, normalized= True)']
